import math
import time
import os
from typing import List

from tqdm import tqdm
import pandas as pd
import numpy as np
import os
os.environ["SM_FRAMEWORK"] = "tf.keras"

import cv2

from . import SaveLoadModel
from . import BatchManager

def get_circle(arr: List):
    """
    Searches related areas and returns a list of dictionaries:
    
    Parameters
    ----------
    arr: Predicted mask

    Returns
    ----------
    list of dictionaries containing information about each related area

    x, y : int8
        Center of gravity

    size : int
        Area

    x_min, y_min, x_len, y_len : int
        Bounding box

    r : float
        Radius of the circle whose area is equal to the found area
    """

    if len(arr) == 0:
        raise ValueError

    inner = np.copy(arr)
    inner[inner != 1] = 0
    N, _, stats, centroids = cv2.connectedComponentsWithStats(inner.astype(np.int8))
    result = []

    for i in range(1, N):
        result.append({
            'x': centroids[i][0],
            'y': centroids[i][1],
            'size': int(stats[i][4]),
            'r': math.sqrt(stats[i][4] / math.pi),
            'x_min': int(stats[i][0]),
            'y_min': int(stats[i][1]),
            'x_len': int(stats[i][2]),
            'y_len': int(stats[i][3]),
        })

    return result

def work(INPUTDIR: str, OUTPUTDIR: str, img_mask: str = ".png", mode: str = 'folder'):
    """
    Runs a neural network on new data:

    Parameters
    ----------   
    INPUTDIR : str
        Path to the folder with images

    OUTPUTDIR : str
        Path to save results

    img_mask : str
        Image suffix to process

    mode : str

        'folder' : Process a folder with images

        'picture' " Process an image
    """

    if INPUTDIR is None:
        INPUTDIR = "./water_drop_detection/NN/data/"

    if OUTPUTDIR is None:
        OUTPUTDIR = "./water_drop_detection/NN/output"

    os.makedirs(OUTPUTDIR, exist_ok = True)

    IMG_SIZE = 32 * 8

    if mode != "folder":
        original = [cv2.imread(os.path.join(INPUTDIR), cv2.IMREAD_COLOR)]
    else:
        original = BatchManager.Open_Images(INPUTDIR, BatchManager.Files_in_dir(INPUTDIR, img_mask))

    img_n = len(original)
    x_train = np.stack(list(map(lambda x: cv2.resize(x, dsize = (IMG_SIZE, IMG_SIZE), interpolation = cv2.INTER_LINEAR), original)))

    model = SaveLoadModel.LoadModel("water_drop_detection/NN/model", "Model")

    start_time = time.time()
    predict = model.predict(x_train)
    print(f"--- predict {time.time() - start_time:.3f} seconds ---")

    count_img = list(range(0, img_n)) # for DataFrame
    areas_drops = []

    for i in tqdm(range(x_train.shape[0])):
        orig = original[i]
        mask = cv2.resize(predict[i], dsize = (orig.shape[1], orig.shape[0]), interpolation = cv2.INTER_CUBIC)
        mask = np.argmax(mask, 2)

        orig[mask == 1] = orig[mask == 1] * [0.5, 0.5, 0.5] + [127, 0, 0]
        circles = get_circle(mask)

        cv2.imwrite(os.path.join(OUTPUTDIR, str(i) + ".png"), orig)

        areas_drops_for_img = []

        for c in circles:
            cv2.rectangle(orig, (c['x_min'], c['y_min']), (c['x_min'] + c['x_len'], c['y_min'] + c['y_len']), (0, 0, 255), 1)

        for c in circles:
            orig = cv2.circle(orig, (int(c['x']), int(c['y'])), int(c['r']), [0, 255, 0], 1)
            orig[int(c['y']), int(c['x'])] = [0, 255, 0]
            areas_drops_for_img.append(c['size'])

        areas_drops.append(areas_drops_for_img)
        cv2.imwrite(os.path.join(OUTPUTDIR, str(i) + "_frame.png"), orig)
        
    df = pd.DataFrame({'Number of image' : count_img, 'Areas of drops': areas_drops})
    df.to_csv(os.path.join(OUTPUTDIR, "Areas.csv"), index = False)


# if __name__ == "__main__":
#     work("./src/NN/data", "./src/NN/output/", ".jpg")



import os
os.environ["SM_FRAMEWORK"] = "tf.keras"

import re
from typing import List

from tqdm import tqdm
import numpy as np
import cv2


def natural_sorting(arr: List[str]):
    """
    Sorts according to the order of the numbers

    Parameters
    ----------
    arr : array of strings

    Returns
    ----------
    Sorted array of strings
    """

    convert = lambda text: int(text) if text.isdigit() else float('inf')
    arr.sort(key = lambda var: [[convert(x), x] for x in re.findall(r'-[0-9]+|.', var)])

    return arr

def Files_in_dir(dir_path: str, mask: str = ""):
    """
    Returns a list of names of all files in alphabetical order containing mask in the name

    Parameters
    ----------
    dir_path : str
        Folder name

    mask : str
        Filter in the file name

    Returns
    ----------
    List of names
    """

    if not os.path.isdir(dir_path):
        raise FileNotFoundError

    return list(filter(lambda x: re.search(mask, x) is not None, natural_sorting(os.listdir(dir_path))))

def Open_Images(path: str, names: List[str]):
    """
    Opens a list of images

    Parameters
    ----------
    path : str
        Folder path

    names : list
        Names of image files

    Returns
    ----------
    Image list
    """

    if not os.path.isdir(path):
        raise FileNotFoundError

    result = []

    for name in names:
        img = cv2.imread(os.path.join(path, name), cv2.IMREAD_COLOR)

        if img is not None:
            result.append(img)

    return result

def SaveBatch(x: np.ndarray, y: np.ndarray, path: str, name: List[str]):
    """
    Stores two numpy arrays

    Parameters
    ----------
    x : image

    y : target

    path : str
        Folder path

    name : str
        File name, for example, 1.npz
    """

    if not os.path.isdir(path):
        raise FileNotFoundError

    np.savez_compressed(os.path.join(path, name), x = x, y = y)

def LoadBatch(path: str, name: str):
    """
    Loads two numpy arrays saved by the SaveBatch function

    Parameters
    ----------
    path : str
        Folder path

    name : str
        File name, for example, 1.npz

    Returns
    ----------
    Two numpy arrays: image and target
    """
  
    if not os.path.isdir(path):
        raise FileNotFoundError
    
    if not os.path.isfile(os.path.join(path, name)):
        raise FileNotFoundError 

    data = np.load(os.path.join(path, name))

    return data['x'], data['y']

def LoadMultipleBatches(path: str, names: List[str]):
    """
    Loads 2 * len(names) numpy arrays saved by the SaveBatch function

    Parameters
    ----------
    path : str
        Folder path

    name : list
        list of names, e.g. ["1.npz", "2.npz"]

    Returns
    ----------
    Two numpy arrays images and targets
    """

    if not os.path.isdir(path):
        raise FileNotFoundError

    size    = 0
    shape_x = None
    shape_y = None
    type_x  = None
    type_y  = None

    for name in tqdm(names):
        x, y = LoadBatch(path, name)
        assert(x.shape[0] == y.shape[0])

        size += x.shape[0]
        assert(shape_x is None or shape_x == x.shape[1:])

        shape_x = x.shape[1:]
        assert(shape_y is None or shape_y == y.shape[1:])

        shape_y = y.shape[1:]
        assert(type_x is None or type_x == x.dtype)

        type_x = x.dtype
        assert(type_y is None or type_y == y.dtype)

        type_y = y.dtype

    result_x = np.empty((size,) + shape_x, dtype = type_x)
    result_y = np.empty((size,) + shape_y, dtype = type_y)
    current = 0

    for name in tqdm(names):
        x, y = LoadBatch(path, name)
        result_x[current : current + x.shape[0]] = x
        result_y[current : current + y.shape[0]] = y
        current += x.shape[0]

    return result_x, result_y



import cv2
import sys
import os

from os.path import dirname, join, abspath
sys.path.insert(0, abspath(join(dirname(__file__), '..')))

from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5 import QtWidgets, QtGui, QtCore
from PyQt5.QtCore import QCoreApplication
from PyQt5.QtGui import QPixmap, QImage
from PyQt5.uic import loadUi
from PyQt5.QtCore import Qt

from NN.TestNetwork import work

def detect_image(filepath, output):
    """Takes the path to the image and the name of the folder to save the result and sends it to the function for detecting drops."""

    work(filepath, output, ".jpg", mode = "picture")

def detect_folder(filepath, output):
    """Takes the path to the folder and the name of the folder to save the result and sends it to the function for detecting drops."""

    work(filepath, output, ".jpg")

class WelcomeScreen(QMainWindow):
    """The class of the original UI screen."""

    def __init__(self, widget):
        super(WelcomeScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page1.ui", self)
        self.widget = widget
        pixmap = QPixmap('water_drop_detection/ui/logo/logobig.png')
        self.label.setPixmap(pixmap)
        self.label_2.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))
        self.start_btn.clicked.connect(lambda: self.start_click())
           
    def start_click(self):
        """Function to go to the next page."""

        select_file = SelectScreen(self.widget) 
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)

class SelectScreen(QMainWindow):
    """Class for the detection object selection screen."""

    def __init__(self, widget):
        super(SelectScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page2.ui", self)
        self.widget = widget
        self.setAcceptDrops(True)
        self.label_2.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))
        self.pushFolderButton.clicked.connect(lambda: self.open_folder())
        self.pushImageButton.clicked.connect(lambda: self.open_image())
    
    def open_folder(self):
        """A function for selecting a folder and going to the next page."""

        file = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select Folder')

        if os.path.exists(file):
            select_file = FolderPretrainScreen(file, self.widget) 
            self.widget.addWidget(select_file)
            self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def open_image(self):
        """A function for selecting an image and going to the next page."""

        file = QtWidgets.QFileDialog.getOpenFileName(self, "Select Image", None, "Image (*.png *.jpg *.jpeg)")[0]

        if "jpeg" in file or "jpg" in file or "png" in file:
            select_file = ImagePretrainScreen(file, self.widget) 
            self.widget.addWidget(select_file)
            self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def dragEnterEvent(self, event):
        """A function that tracks the transfer of an image/folder to a widget."""

        if event.mimeData().hasImage:
            event.accept()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        """A function that tracks the transfer of an image/folder to a widget."""

        if event.mimeData().hasImage:
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        """A function that captures the path of the image/folder on which drops want to be detected."""

        if event.mimeData().hasImage:
            event.setDropAction(Qt.CopyAction)
            self.file_path = event.mimeData().urls()[0].toLocalFile()
            self.set_image(self.file_path)
            event.accept()
        else:
            event.ignore()

    def set_image(self, file):
        """Function to move to the next slide if the image/folder has been moved to the widget."""

        if "jpeg" in file or "jpg" in file or "png" in file:
            select_file = ImagePretrainScreen(file, self.widget) 
            self.widget.addWidget(select_file)
            self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        else:
            select_file = FolderPretrainScreen(file, self.widget) 
            self.widget.addWidget(select_file)
            self.widget.setCurrentIndex(self.widget.currentIndex() + 1)

class ImagePretrainScreen(QMainWindow):
    """A class for the image display screen before detection."""

    def __init__(self, file, widget):
        super(ImagePretrainScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page3_image.ui", self)
        self.file = file
        self.widget = widget
        self.pixmap = QPixmap(file)
        self.label_2.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))

        super().layout().activate()
        self.label.setPixmap(self.pixmap.scaled(1700, 717, QtCore.Qt.KeepAspectRatio))
        self.toNeuralButton.clicked.connect(lambda: self.findDrops(file))
        self.ImageProcessingButton.clicked.connect(lambda: self.ImageProcessing(file))
        self.BackButton.clicked.connect(lambda: self.Back())

    def findDrops(self, file):
        """Function for switching to the detection slide."""

        select_file = FolderResultScreen(file, 0, self.widget) 
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def ImageProcessing(self, file):
        """Function for switching to the image processing slide."""

        select_file = EditScreen(file, self.widget)
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def Back(self):
        """Function to return to the previous slide."""

        select_file = SelectScreen(self.widget)
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)

class FolderPretrainScreen(QMainWindow):
    """A class for displaying the folder name before detecting it."""
    
    def __init__(self, file, widget):
        super(FolderPretrainScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page3_folder.ui", self)
        self.widget = widget
        self.label.setText("Your choice: \n" + file)
        self.label_3.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))
        print(file)
        
        self.toNeuralButton.clicked.connect(lambda: self.findDrops(file))
        self.BackButton.clicked.connect(lambda: self.Back())
    
    def findDrops(self, file):
        """Function for switching to the detection slide."""

        select_file = FolderResultScreen(file, 1, self.widget) 
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def Back(self):
        """Function to return to the previous slide."""

        select_file = SelectScreen(self.widget) 
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)

class EditScreen(QMainWindow):
    """A class for the input image processing screen."""

    def __init__(self, file, widget):
        super(EditScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page4_image.ui", self)
        self.rotation = 0
        self.widget = widget
        self.pixmap = QPixmap(file)
        self.photos = [QPixmap(file)]
        self.edit_pixmap = QPixmap(file)
        self.label_2.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))
        self.label.setPixmap(QPixmap(file).scaled(1700, 717, QtCore.Qt.KeepAspectRatio))
        
        self.okButton.clicked.connect(lambda: self.OK_button(file))
        self.backButton.clicked.connect(lambda: self.Back_button())
        self.rotateButton.clicked.connect(lambda: self.Rotate_button())
 
    def OK_button(self, file):
        """Function to switch to the detection slide."""

        self.photos[-1].save('pixmap.png')
        select_file = FolderResultScreen(file, 0, self.widget) 
        self.widget.addWidget(select_file)
        self.widget.setCurrentIndex(self.widget.currentIndex() + 1)
        
    def Back_button(self):
        """A function to cancel the last processing action."""

        if len(self.photos) > 1:
            self.photos.pop()
            self.label.setPixmap(self.photos[-1].scaled(1700, 717, QtCore.Qt.KeepAspectRatio))
    
    def Rotate_button(self):
        """Function for rotating the image by 90 degrees."""

        pixmap = self.edit_pixmap.copy()
        self.rotation += 90
        transform = QtGui.QTransform().rotate(90)
        pixmap = pixmap.transformed(transform, QtCore.Qt.SmoothTransformation)
        self.label.setPixmap(pixmap.scaled(1700, 717, QtCore.Qt.KeepAspectRatio))
        self.edit_pixmap = pixmap
        self.photos.append(pixmap)
        pixmap.save('pixmap.png')
        
    def mousePressEvent(self, eventQMouseEvent):
        """A function for tracking mouse clicks on a widget with an image."""

        offset = self.label.pos()
        self.label.originQPoint = eventQMouseEvent.pos() - offset
        self.label.currentQRubberBand = QtWidgets.QRubberBand(QtWidgets.QRubberBand.Rectangle, self.label)
        self.label.currentQRubberBand.setGeometry(QtCore.QRect(self.label.originQPoint, QtCore.QSize()))
        self.label.currentQRubberBand.show()

    def mouseMoveEvent(self, eventQMouseEvent):
        """A function for tracking the movement of the previously pressed mouse button 
        on the widget with the image and building a rectangle for cropping."""

        offset = self.label.pos()
        self.label.currentQRubberBand.setGeometry(QtCore.QRect(self.label.originQPoint, eventQMouseEvent.pos() - offset).normalized())

    def mouseReleaseEvent(self, eventQMouseEvent):
        """A function for cropping the image along the resulting rectangle after pressing the mouse button."""

        self.label.currentQRubberBand.hide()
        currentQRect = self.label.currentQRubberBand.geometry()
        self.label.currentQRubberBand.deleteLater()
        self.edit_pixmap = self.label.grab(currentQRect)
        self.photos.append(self.edit_pixmap)
        self.label.setPixmap(self.edit_pixmap.scaled(self.label.size(), QtCore.Qt.KeepAspectRatio))
        self.edit_pixmap.save('pixmap.png')

class FolderResultScreen(QMainWindow):
    """A class for the detection result display screen."""

    def __init__(self, input_folder, flag, widget):
        super(FolderResultScreen,self).__init__()

        loadUi("water_drop_detection/ui/ui/page5_folder.ui", self)
        self.widget = widget
        self.label_4.setPixmap(QPixmap('water_drop_detection/ui/logo/wallpaper1.jpg'))

        if not flag:

            self.output_folder = input_folder[:len(input_folder) - 1 - input_folder[::-1].index('/') + 1] \
                + "detected_"+ input_folder[len(input_folder) - 1 - input_folder[::-1].index('/') + 1:input_folder.find(".")]
            
            if os.path.exists("pixmap.png"):
                detect_image("pixmap.png", self.output_folder)
                os.remove("pixmap.png")
            else:
                detect_image(input_folder, self.output_folder)

        else:
            self.output_folder = input_folder + '/detected'
            detect_folder(input_folder, self.output_folder)

        self.mode = 0
        self.index = 0
        self.img_no_frame = []
        self.img_with_frame = []

        dirr = sorted(os.listdir(self.output_folder))

        for filename in dirr:
            img = cv2.imread(os.path.join(self.output_folder,filename))
            
            if img is not None and "frame" not in filename:
                self.img_no_frame.append(img)
                self.img_with_frame.append(cv2.imread(os.path.join(self.output_folder, filename[:filename.find(".")] \
                                                                   + "_frame" + filename[filename.find("."):])))

        if len(self.img_no_frame[self.index].shape) < 3:
            frame = cv2.cvtColor(self.img_no_frame[self.index], cv2.COLOR_GRAY2RGB)
        else:
            frame = cv2.cvtColor(self.img_no_frame[self.index], cv2.COLOR_BGR2RGB)
            
        h, w = self.img_no_frame[self.index].shape[:2]
        bytesPerLine = 3 * w
        qimage = QImage(frame.data, w, h, bytesPerLine, QImage.Format.Format_RGB888) 
        pix = QPixmap(qimage)
        self.label.setPixmap(pix.scaled(1700, 650, QtCore.Qt.KeepAspectRatio))
        
        self.label_3.setText("Result saved in " + self.output_folder)
            
        self.nextButton.clicked.connect(lambda: self.nextPhoto(self.img_no_frame) \
                                        if self.mode == 0 else self.nextPhoto(self.img_with_frame))
        
        self.quitButton.clicked.connect(lambda: QApplication.closeAllWindows())
        self.withoutFrame.clicked.connect(lambda: self.changeMode(0))
        self.withFrame.clicked.connect(lambda: self.changeMode(1))

    def changeMode(self, flag):
        """Function for changing the result viewing mode."""

        if(self.mode != flag):
            self.mode = not(self.mode)

        if (self.mode):
            self.nextPhoto(self.img_with_frame, mode = True)
        else:
            self.nextPhoto(self.img_no_frame, mode = True)

    def nextPhoto(self, images, mode = False):
        """Function to view the next image, if available."""

        if not mode:
            self.index += 1

        if self.index >= len(images):
            self.index = 0

        if len(images[self.index].shape) < 3:
            frame = cv2.cvtColor(images[self.index], cv2.COLOR_GRAY2RGB)
        else:
            frame = cv2.cvtColor(images[self.index], cv2.COLOR_BGR2RGB)

        h, w = images[self.index].shape[:2]
        bytesPerLine = 3 * w
        qimage = QImage(frame.data, w, h, bytesPerLine, QImage.Format.Format_RGB888) 
        pix = QPixmap(qimage)
        self.label.setPixmap(pix.scaled(1700, 650, QtCore.Qt.KeepAspectRatio))
                
def main():
    """A function to launch the start screen."""

    app = QCoreApplication.instance()

    if app is None:
        app = QApplication(sys.argv)
        widget = QtWidgets.QStackedWidget()
        welcome = WelcomeScreen(widget)
        widget.addWidget(welcome)
        widget.showMaximized()

    try:
        sys.exit(app.exec_())
    except:
        print("Exiting")

    



import os
os.environ["SM_FRAMEWORK"] = "tf.keras"

import sys
from typing import List

from os.path import dirname, join, abspath
sys.path.insert(0, abspath(join(dirname(__file__), '..')))

import matplotlib.pyplot as plt
import numpy as np
import cv2

import water_drop_detection.NN.BatchManager as BatchManager


class DropDataset():

    def __init__(self, folder: str, data_postfix: str, mask_postfix: str, transform = None):
        """
        Shows 10 images:

        Parameters
        ---------- 
        folder : str
            The path to the folder with the dataset

        data_postfix : str
            End of files that will be images

        mask_postfix : str
            End of files that will be masks

        transform: Transformations applied to the input image
        """

        if not os.path.isdir(folder):
            raise FileNotFoundError
             
        images = sorted([os.path.join(folder, name) for name
                         in os.listdir(folder) if name.endswith(data_postfix)])

        masks = sorted([os.path.join(folder, name) for name
                        in os.listdir(folder) if name.endswith(mask_postfix)])
        
        if len(images) == 0 or len(masks) == 0:
            raise FileNotFoundError
        
        if len(images) != len(masks):
            raise Exception("The number of images and masks does not match")
        

        self.pairs = list(zip(images, masks))

    def __len__(self):
        """Returns the number of elements"""

        return len(self.pairs)

    def __getitem__(self, index: int):
        """
        Load image and mask:

        Parameters
        ---------- 
        index : int
            Pair number

        Returns
        ----------
        Image and mask
        """

        # file path tuple
        img, mask = self.pairs[index]

        # read data as numpy array
        img = cv2.imread(img, cv2.IMREAD_COLOR)
        mask = np.load(mask, allow_pickle = True)

        return img, mask

def visualize_samples(dataset: DropDataset, indices: List[int], title: str = None, count: int = 10):
    """
    Shows 10 images:

    Parameters
    ----------
    dataset : an instance of the DropDataset class

    indices : list
        Order of indices to display

    title : str
        Image title

    count : int
        The number to display
    """

    # visualize random 10 samples
    plt.figure(figsize = (count * 3, 6))
    display_indices = indices[:count]

    if title:
        plt.suptitle("%s %s/%s" % (title, len(display_indices), len(indices)))

    for i, index in enumerate(display_indices):
        x, y = dataset[index]
        plt.subplot(2, count, i + 1)
        plt.imshow(x.squeeze())
        plt.grid(False)
        plt.axis('off')
        plt.subplot(2, count, i + 1 + count)
        plt.imshow(y.squeeze())
        plt.grid(False)
        plt.axis('off')

    plt.show()

IMG_SIZE = 32 * 8

def SaveToFile(dataset: DropDataset, resultDir: str, name: str):
    """
    Changes the size, convert the mask to binary and save

    Parameters
    ----------
    dataset : an instance of the DropDataset class

    resultDir : str
        The path to the file to save the image

    name : str
        Image Name
    """

    im_array = []
    mask_array = []

    for im, mask in dataset:
        im = cv2.resize(im, dsize = (IMG_SIZE, IMG_SIZE), interpolation = cv2.INTER_LINEAR)
        mask = mask.astype("uint8")
        mask = cv2.resize(mask, dsize = (IMG_SIZE, IMG_SIZE), interpolation = cv2.INTER_NEAREST)
        mask_hot_one = np.empty((IMG_SIZE, IMG_SIZE, 2))
        mask_hot_one[mask == 0] = [1, 0]
        mask_hot_one[mask == 1] = [0, 1]
        im_array.append(im)
        mask_array.append(mask_hot_one)

    BatchManager.SaveBatch(np.stack(im_array), np.stack(mask_array), resultDir, name)

# def main():
#     orig_dataset = DropDataset("./src/NN/data/", ".jpg", "_mask.jpg.npy")
#     SaveToFile(orig_dataset, "./src/NN/batch/orig", "orig")

#     # orig_dataset = DropDataset("../data/C/data2/", ".png", "_mask.npy")
#     # SaveToFile(orig_dataset, "../batch/orig", "orig2")

#     indices = np.random.choice(np.arange(len(orig_dataset)), 7, replace=False)
#     visualize_samples(orig_dataset, indices, "Samples")

# if __name__ == "__main__":
#     main()



import random
import os
os.environ["SM_FRAMEWORK"] = "tf.keras"

import segmentation_models as sm
import tensorflow as tf
import numpy as np
import keras

from keras.callbacks import CSVLogger

import SaveLoadModel
import BatchManager

keras.backend.set_image_data_format('channels_last')


def main():
    BACKBONE = 'resnet50'
    BATCH_DIR = "./water_drop_detection/NN/batch/"
    MODEL_DIR = "./water_drop_detection/NN/model/"
    MODEL_NAME = "Model"
    BATCH_SIZE = 15
    LR = 3E-5
    EPOCHS = 450
    VERBOSE = 1
    NEW_MODEL = False
    NAMES = []

    preprocess_input = sm.get_preprocessing(BACKBONE)

    def List_batch_names(batch_name):
        """
        Returns a list of paths to batches

        Parameters
        ----------
        batch_name : str
            Batch folder name

        Returns
        ----------
        List of batch paths
        """

        return [os.path.join(batch_name, x) for x in BatchManager.Files_in_dir(os.path.join(BATCH_DIR, batch_name))]
    
    NAMES += List_batch_names("orig")

    x_train, y_train = BatchManager.LoadMultipleBatches(BATCH_DIR, NAMES)
    x_train = preprocess_input(x_train)

    if NEW_MODEL:
        model = sm.Unet(BACKBONE, classes = 2, activation = 'softmax', encoder_weights = 'imagenet')
    else:
        load_name = MODEL_NAME
        model = SaveLoadModel.LoadModel(MODEL_DIR, load_name)

    model.compile(
        optimizer = tf.keras.optimizers.Adam(LR),
        loss = sm.losses.bce_jaccard_loss,
        metrics = [sm.metrics.iou_score],
    )

    csv_logger = CSVLogger(MODEL_DIR + MODEL_NAME + ".log")
    save_model_val_best = tf.keras.callbacks.ModelCheckpoint(
        filepath = MODEL_DIR + MODEL_NAME + '_best.h5',
        save_weights_only = True,
        monitor = 'iou_score',
        mode = 'max', 
        save_best_only =True)

    with open(MODEL_DIR + MODEL_NAME + "_best.json", "w") as json_file:
        json_file.write(model.to_json())

    def rotation_flip(img, seed):
        """
        Rotates and/or reflects the image

        Parameters
        ----------
        img : numpy array

        seed : action 
            * seed % 4 number of 90 degree rotations
            * (seed // 4) % 2 - presence of reflection
            * (0 1 2 3) - without reflection
            * (4 5 6 7) - with reflection

        Returns
        ----------
        Image
        """

        r = seed % 4
        f = (seed // 4) % 2

        if f:
            return np.rot90(np.flipud(img), r)
        else:
            return np.rot90(img, r)

    def my_generator_indexes(trainX, trainY, batch_size, indexes):
        """
        Used in fit_generator

        Parameters
        ----------
        trainX : numpy array of images - datasetX
        trainY : numpy array of images - datasetY
        batch_size : size of returned batch
        indexes : allowed indexes

        Returns
        ----------
        Two numpy arrays with images and masks 
        """

        sx = (batch_size,) + trainX.shape[1:]
        resultX = np.empty(sx, dtype = trainX.dtype)
        sy = (batch_size,) + trainY.shape[1:-1] + (2,)
        resultY = np.empty(sy, dtype = trainY.dtype)

        while True:
            for i in range(batch_size):
                index = np.random.choice(indexes)
                seed = random.randrange(8)
                resultX[i] = rotation_flip(trainX[index], seed)
                resultY[i] = rotation_flip(trainY[index], seed)

            yield resultX, resultY

    indexes_train = np.arange(len(x_train))
    model.fit_generator(
        generator = my_generator_indexes(x_train, y_train, BATCH_SIZE, indexes_train),
        steps_per_epoch = len(indexes_train) // BATCH_SIZE,
        epochs = EPOCHS,
        verbose = VERBOSE,
        callbacks = [csv_logger, save_model_val_best])

    SaveLoadModel.SaveModel(model, MODEL_DIR, MODEL_NAME)


if __name__ == "__main__":
    main()



import os
os.environ["SM_FRAMEWORK"] = "tf.keras"

from keras.models import model_from_json

def SaveModel(model, path: str, name: str):
    """
    Saves model and weights:

    Parameters
    ----------   
    model: segmentation_models.Unet
        The model to save

    path : str
        The path to save the model

    name : str
        File name without extension
    """

    with open(os.path.join(path, name + ".json"), "w") as json_file:
        model_json = model.to_json()
        json_file.write(model_json)

    model.save_weights(os.path.join(path, name + ".h5"))

def LoadModel(path, name: str, weights = None):
    """
    Load model and weights:

    Parameters
    ----------
    path : str
        The path to the model

    name : str
        File name without extension

    weights: str
        Weight name (if not the same as 'name')

    Returns
    ----------
    segmentation_models.Unet model with loaded weights
    """

    if weights is None:
        weights = name

    try:
        with open(os.path.join(path, name + ".json"), "r") as json_file:
            loaded_model_json = json_file.read()
            model = model_from_json(loaded_model_json)

        model.load_weights(os.path.join(path, weights + ".h5"))

    except FileNotFoundError:
        raise

    return model



import setuptools

with open("README.md", "r", encoding = "utf-8") as file:
    read_me = file.read()

with open("requirements.txt", "r") as file:
    requirements = file.read().splitlines()

setuptools.setup(
    name = "water_drop_detection",
    version = "0.7",
    author = "Aleksandr Leisle, Sofia Volodina, Alina Shitenko",
    author_email = "a.leisle@g.nsu.ru, s.volodina@g.nsu.ru, a.shitenko@g.nsu.ru",
    description = "A neural network that detects water drops on the surface from an image.",
    long_description = read_me,
    long_description_content_type = "text/markdown",
    project_urls = {
        'Documentation': 'https://github.com/HerrPhoton/Water_drop_detection/blob/master/Documentation.pdf',
        'Bug Tracker': 'https://github.com/HerrPhoton/Water_drop_detection/issues',
        'Homepage': 'https://github.com/HerrPhoton/Water_drop_detection',
    },
    packages = ['water_drop_detection', 'water_drop_detection/NN', 'water_drop_detection/ui'],
    install_requires = requirements,
    classifiers = [
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
    ],
    python_requires = '==3.9.*',
)
